package Cvtree;

#This is a OO wrapper for the tool CVTree
#CVTree is used to calculate the distance between 2 genomes
#CVTree requires the .faa file (protein sequences) for each genome

use strict;
use warnings;
our $AUTOLOAD;    # before Perl 5.6.0 say "use vars '$AUTOLOAD';"
use Carp;

use Cwd;
use File::Temp qw/ tempfile :mktemp/;


use Buster;


my @FIELDS = qw(quiet buster tmp_dir node);

# The constructor for the class
sub new {
	my ( $class, %arg ) = @_;

	#Bless an anonymous empty hash
	my $self = bless {}, $class;

	#Fill all of the keys with the fields
	foreach (@FIELDS) {
		$self->{$_} = undef;
	}
	
	$self->tmp_dir('/home/mlangill/common/research/tmp');
	
	#Set each attribute that is given as an arguement
	foreach ( keys(%arg) ) {
		$self->$_( $arg{$_} );
	}

	return $self;
}

sub run {
	my ( $self, $seq1, $seq2 ) = @_;
	my $tmp_dir = $self->tmp_dir;
	
	#my $start_dir = getcwd();
	#chdir($tmp_dir) or croak "Couldn't change directory to: $tmp_dir";
	
	#Get the filenames without suffixes
	my $seq1a=path2filename($seq1);
	my $seq2a=path2filename($seq2);
	
	#make the filename with suffix
	my $seq1b = $seq1a . ".faa";
	my $seq2b = $seq2a . ".faa";
	
	#create the 2 file
	symlink $seq1, "$tmp_dir/$seq1b";
	symlink $seq2, "$tmp_dir/$seq2b";
	
	#Create the input file
	my($fhin, $input_filename) = tempfile("$tmp_dir/cvinXXXXXX");
	
	print $fhin "2\n";
	print $fhin "$seq1a 1\n";
	print $fhin "$seq2a 2\n";
	
	close $fhin;
	
	#Create the output filename
	my $output_filename = mktemp("$tmp_dir/cvoutXXXXXX");
	
	my $command="./cvtree_simple.pl $tmp_dir $input_filename $output_filename";

    if($self->quiet){
        open(STDERR,">/dev/null");
    }

    #If Buster flag is set, get a free node and run it on that node
    if($self->buster){
        my $free_node;
        if($self->node){
	       $free_node = $self->node;
        }else{
           my $buster_obj= new Buster();
	       my $free_node= $buster_obj->get_node(); 
        }
        #$free_node = node_format($free_node);
        $command = "clsh -q $free_node ".$command;
        
    }

    #run the actual command
    unless (open(CVTREE, "$command |")){
	    croak "Cannot run $command";
	}

    chomp(my @stdout=<CVTREE>);

    if($self->quiet){
	close(STDERR);
    }
    #parse the output file
	#my $result = get_results($output_filename);
	my ($result) = @stdout;
	unlink $output_filename,$input_filename,$seq1b,$seq2b;
	
	#change back to proper directory
	#chdir($start_dir) or croak "Couldn't change directory to: $start_dir";
	
	return $result;

}

sub get_results{
	my ($filename) =@_;
	#$output_filename =~ s/home/home.westgrid/;
	open (OUTPUT,'<',$filename) or croak "Can't open results file: $filename $!";
	<OUTPUT>;
	my $line = <OUTPUT>;
	close(OUTPUT);
	chomp ($line);
	if($line =~ /\S+ \S+ (\S+)/){
		return $1;
	}else{
		croak "Can't parse results file: $filename";
	}

}

#Takes a filename that may include a path too it and returns only the filename with no extension
sub path2filename{
    my ($filename)=@_;
    if( $filename =~ /(.*\/)*(\w+)(\.\w+)?/){
	return $2;
    }else{
	die "ERROR: Can't extract filename from path: $!";
	return;
    }

}

# This takes the place of methods to set or get the value of an attribute
sub AUTOLOAD {
	my ( $self, $newvalue ) = @_;

	#get the unknown method call
	my $attr = $AUTOLOAD;

	#Keep only the method nam
	$attr =~ s/.*:://;

	#Die if the key does not already exist in the hash
	unless ( exists( $self->{$attr} ) ) {
		croak "No such attribute '$attr' exists in the class ";
	}

	# Turn off strict references to enable "magic" AUTOLOAD speedup
	no strict 'refs';

	# define subroutine
	*{$AUTOLOAD} = sub {
		my ( $self, $newvalue ) = @_;
		$self->{$attr} = $newvalue if defined($newvalue);
		return $self->{$attr};
	};

	# Turn strict references back on
	use strict 'refs';

	#Set the new value for the attribute if available
	$self->{$attr} = $newvalue if defined($newvalue);

	#Always return the current value for the attribute
	return $self->{$attr};
}

sub node_format{
    my $node = shift;
    while(!($node =~ /\d\d\d/)){
        $node = 0 . $node;
    }
    $node = 'c'.$node;
    return $node;
    
}

sub DESTROY {
}

1;

=head1 NAME

Mauve: Used to run the Mauve alignment tool

=head1 Synopsis
 use Mauve;

 my $mauve_obj = new Mauve();

 my $mauve_results_obj = $mauve_obj->run($seqfile1,$seqfile2);

 my @mauve_alignment=@{$mauve_results_obj->output_alignment()};


=head1 AUTHOR

Morgan Langille

=head1 Date

May 23th, 2006

=cut

