package Genome_Picker;

#This is meant as a starting point for programming new object modules.

use strict;
use warnings;
use Carp;

our $AUTOLOAD;

my @_cutoffs = qw(
max_cutoff
min_cutoff
min_compare_cutoff
max_dist_single_cutoff
min_dist_single_cutoff
); 

my @FIELDS = (@_cutoffs, 'microbedb_version', 'rep_accnums');

use lib "../microbedb/MicrobeDB";
use Search;


sub new {
	my ( $class, %arg ) = @_;

	#Bless an anonymous empty hash
	my $self = bless {}, $class;

	#Fill all of the keys with the fields
	foreach (@FIELDS) {
		$self->{$_} = undef;
	}

	#Set some defaults

	#Set each attribute that is given as an arguement
	foreach ( keys(%arg) ) {
		$self->$_( $arg{$_} );
	}
    my $ver = $self->microbedb_version;
    #Get a list of all rep_accnum for the given version
    my $sobj    = new Search();
    my @results = $sobj->object_search(new Replicon(version_id => $ver));
	my %rep_accnum_lookup;
	foreach(@results){
	    $rep_accnum_lookup{$_->rep_accnum()}=1;
	}
	$self->rep_accnums(\%rep_accnum_lookup);

	return $self;
}



#INPUT: a replicon
#OUPTUT: array of replicons within the proper evolutionary distance
sub find_comparative_genomes {
	my ($self,$rep_obj) = @_;
    
    my $rpv_id = $rep_obj->rpv_id();
    
    my $rp =420;
	my @results = $self->_max_filter($rep_obj);
   
	@results = $self->_remove_same_strain( $rep_obj, \@results );
	
	if($rpv_id == $rp){
	    print "After Max Filter\n";
       $self->_print_distances(@results);
    } 

	@results = $self->_remove_genomes_close_to_query( \@results );
	if($rpv_id == $rp){
	    print "\n After Min Filter to query\n";
       $self->_print_distances(@results);
    } 

	@results = $self->_remove_similar_genomes( \@results );
	if($rpv_id == $rp){
	     print "\n After Min Filter to all\n";
       $self->_print_distances(@results);
    } 

	#Look for a single genome that meets the $MAX_DIST_SINGLE_CUTOFF, otherwise return undef
	my $found = 0;
	foreach (@results) {
		if ( $_->{distance} <= $self->max_dist_single_cutoff() ) {
			$found = 1;
			last;
		}

	}
	return unless $found;

	#Look for a single genome that meets the $MIN_DIST_SINGLE_CUTOFF, otherwise return undef
	$found = 0;
	foreach (@results) {
		if ( $_->{distance} >= $self->min_dist_single_cutoff() ) {
			$found = 1;
			last;
		}

	}
	return unless $found;

	if ( scalar(@results) >= $self->min_compare_cutoff() ) {
		print "\nquery:\t";
		print_rep_info($rep_obj);
		foreach my $curr_result (@results) {
			my $dist= sprintf( "%.4f", $curr_result->{distance} );
			print "$dist\t";
			print_rep_info(new Replicon(rep_accnum =>$curr_result->{rep_accnum2}, version_id =>$rep_obj->version_id()));
		}
	}
	
	#get the replicon objects
	my @good_reps;
	my $sobj = new Search();
	if ( scalar(@results) >= $self->min_compare_cutoff() ) {
		foreach (@results) {
			my $temp_rep = new Replicon( rep_accnum => $_->{rep_accnum2}, version_id =>$rep_obj->version_id() );
			push( @good_reps, $sobj->object_search($temp_rep) );
		}
	} else {
		return;
	}
	return @good_reps;
}

#Input: Replicon object
#Gets all the replicons within a certain max distance away
sub _max_filter {
	my ($self,$rep_obj) = @_;

	my $max_cutoff=$self->max_cutoff();
	my $rpv_id  = $rep_obj->rpv_id;
	my $accnum = $rep_obj->rep_accnum;

	my $sobj    = new Search( advanced_where => 1 );
	my @results = $sobj->table_search(
		'replicon_distance',
		{
			'rep_accnum1'  => "= $accnum",
			'distance' => "< $max_cutoff"
		}
	);

	#the id may be in the first or second column in the table so we have to search twice
	my @temp = $sobj->table_search(
		'replicon_distance',
		{
			'rep_accnum2'  => "= $accnum",
			'distance' => "< $max_cutoff"
		}
	);

	#Make all the records have the id that we searched with in the first column (rpv_id1)
	foreach (@temp) {
		push(
			@results,
			{
				'rep_accnum1'  => $_->{rep_accnum2},
				'rep_accnum2'  => $_->{rep_accnum1},
				'distance' => $_->{distance}
			}
		);
	}
	
	#Check to make sure all rep_accnums are from this version
	my $rep_accnum_lookup= $self->rep_accnums();
	@temp =();
	foreach(@results){
	    my $rep_accnum = $_->{rep_accnum2};
	    if($rep_accnum_lookup->{$rep_accnum}){
	        push(@temp,$_);
	    }
	}
	@results=@temp;
	
	
	return @results;
}

sub _remove_genomes_close_to_query {
	my ($self,$results) = @_;
	
	my $min_cutoff = $self->min_cutoff();
	
	#filter for min_cutoff
	my @temp;
	foreach (@$results) {
		if ( $_->{distance} > $min_cutoff ) {
			push( @temp, $_ );
		}
	}

	return @temp;
}

#Filter out genomes that are for the exact same strain (but just from different sequencing centres)
sub _remove_same_strain {
	my ($self,$rep_obj,$results) = @_;

	my $sobj = new Search();
	my @temp;
	foreach (@$results) {
		my ($tmp_rep) = $sobj->object_search( new Replicon( rep_accnum => $_->{rep_accnum2},version_id =>$rep_obj->version_id() ) );
		
		if(!defined($tmp_rep)){
		    next;
		}
		unless ( $tmp_rep->definition() eq $rep_obj->definition() ) {
			push( @temp, $_ );
		}
	}
	return @temp;
}

#Removes any genome that is too close (ie. closer than min_cutoff) to another reference genome
#Tries to remove the ones that are close to the most other genomes (i.e removes the hub genome of a wheel)
#
sub _remove_similar_genomes {
	my ($self,$results) = @_;;

	my $min_compare_cutoff = $self->min_compare_cutoff();
	my $min_cutoff = $self->min_cutoff();
	
	#put all replicon ids into a single array
	my @candidate_genomes;
	foreach (@$results) {
		my $tmp;
		$tmp->{rep_accnum} = $_->{rep_accnum2};
		push( @candidate_genomes, $tmp );
	}

	#As long as there is enough candidate genomes left
  MAJOR_LOOP:
	while ( scalar(@candidate_genomes) > $min_compare_cutoff ) {
		my $total = scalar(@candidate_genomes);

		#Intialize bad and good numbers
		foreach (@candidate_genomes) {
			$_->{good} = 0;
			$_->{bad}  = 0;
		}

		#Go through each distance
		for my $i ( 0 .. $total ) {
			for my $j ( ( $i + 1 ) .. ( $total - 1 ) ) {

				if ( $self->_get_distance( $candidate_genomes[$i]->{rep_accnum}, $candidate_genomes[$j]->{rep_accnum} ) < $min_cutoff )
				{
					$candidate_genomes[$i]->{bad}++;
					$candidate_genomes[$j]->{bad}++;
				} else {
					$candidate_genomes[$i]->{good}++;
					$candidate_genomes[$j]->{good}++;
				}
			}
		}

		#sort through and find the one with the highest number of "bad" connections
		my $high_bad = 0;
		my $bad_genome;
		foreach (@candidate_genomes) {
			if ( $_->{bad} > $high_bad ) {
				$bad_genome = $_->{rep_accnum};
				$high_bad   = $_->{bad};
			}
		}

		#No more bad genomes found so stop looking
		if ( $high_bad == 0 ) {
			last MAJOR_LOOP;
		}

		#Remove the "bad" one
		for my $i ( 0 .. $total ) {
			if ( $bad_genome eq $candidate_genomes[$i]->{rep_accnum} ) {
				splice( @candidate_genomes, $i, 1 );
				last;
			}
		}
	}

	#filter out the bad ones from the input results
	my @filtered_results;
	foreach my $curr_rep (@$results) {
		foreach my $curr_candidate (@candidate_genomes) {
			if ( $curr_rep->{rep_accnum2} eq $curr_candidate->{rep_accnum} ) {
				push( @filtered_results, $curr_rep );
				last;
			}
		}
	}

	return @filtered_results;
}

#Input: 2 rep_accnums from microbeDB
#Output: distance between replicons
sub _get_distance {
	my ($self,$rep_accnum1,$rep_accnum2) = @_;

	my $sobj = new Search();
	my ($record) = $sobj->table_search(
		'replicon_distance',
		{
			'rep_accnum1' => $rep_accnum1,
			'rep_accnum2' => $rep_accnum2
		}
	);
	unless ( defined($record) ) {
		($record) = $sobj->table_search(
			'replicon_distance',
			{
				'rep_accnum1' => $rep_accnum2,
				'rep_accnum2' => $rep_accnum1
			}
		);
	}
	unless($record->{distance}){

	    die "Could not find distance for given replicon ids: $rep_accnum1, $rep_accnum2!";
	}

	return $record->{distance};
}

#print object
sub print_object{
	my ($self) =@_;
	foreach my $curr_key ($self->all_fields()){
		my $print_line = join("\t", $curr_key, $self->$curr_key());
		print "$print_line\n";
	}
}

#returns an array of all field names for this class
sub all_fields{
  my ($self) = @_;
  return keys(%$self);
}

# This takes the place of methods to set or get the value of an attribute
sub AUTOLOAD {
    my ($self,$newvalue) = @_;
    #get the unknown method call
    my $attr = $AUTOLOAD;

    #Keep only the method name
    $attr =~ s/.*:://;    

    #Die if the key does not already exist in the hash
    unless (exists($self->{$attr})){
	croak "No such attribute '$attr' exists in the class ";
    }

    # Turn off strict references to enable "magic" AUTOLOAD speedup
    no strict 'refs';

    # define subroutine
    *{$AUTOLOAD} = sub {my($self,$newvalue)=@_;
			$self->{$attr}=$newvalue if defined($newvalue);
			return $self->{$attr}};    

    # Turn strict references back on
    use strict 'refs';

    #Set the new value for the attribute if available
    $self->{$attr} = $newvalue if defined($newvalue);

    #Always return the current value for the attribute
    return $self->{$attr};
}

#input is a Replicon object
sub print_rep_info {
	my $rep_obj = shift;
	my $search_obj = new Search();
	if(defined($rep_obj->rpv_id())){
	    ($rep_obj)= $search_obj->object_search(new Replicon(rpv_id => $rep_obj->rpv_id()));
	}else{
	    ($rep_obj)= $search_obj->object_search($rep_obj);
	}
	
	
	my $delimiter ="\t";
	
	my @to_print;
	push(@to_print,$rep_obj->rpv_id(),$rep_obj->definition(),$rep_obj->rep_size());
	
	#Fetch the genome project that is associated with this replicon
	$search_obj->return_obj('GenomeProject');
	my ($g_obj)= $search_obj->object_search($rep_obj);
	
	#push(@to_print,$g_obj->gram_stain(),$g_obj->genome_gc(),$g_obj->patho_status());
	push(@to_print,$g_obj->genome_gc(),$g_obj->patho_status());
	my $print_str = join($delimiter,@to_print);
	
	print "$print_str\n";
	
	
}

sub _print_distances{
    my ($self,@distances) =@_;
    my $sobj = new Search();
    my $version = $self->microbedb_version;
    foreach(@distances){
        my ($rep) = $sobj->object_search(new Replicon(version_id => $version, rep_accnum => $_->{rep_accnum2}));    
        my $def = $rep->definition();
        my $acc = $rep->rep_accnum();
        my $dist = $_->{distance};
        print "$dist\; $acc\; $def \n";
    }
}


#

#Anything put in this method will be run when the object is destroyed
sub DESTROY{
}


1;
